import { FC, ComponentProps } from 'react';
import { Column as ReactTableColumn, Row, SortingRule } from 'react-table';
import { CSSWithSelectors } from '@cloudflare/style-container';
import { Div } from '@cloudflare/elements';
import { Omit } from '@cloudflare/types';

export type Column<Data extends object = {}> = {
  isDragHandleVisible?: () => boolean;
  getCellData?: () => any;
  enableSortBy?: boolean;
} & ReactTableColumn<Data>;

type ExpandableProps<Data extends object = {}> = {
  row: Row<Data>;
  maxWidth?: string | number;
};
type ExpandableComponent<Data extends object = {}> = FC<ExpandableProps<Data>>;

export type RowProps<Data extends object = {}> = {
  row: Row<Data>;
  rowId: string;
  rowStyle?: ComponentProps<typeof Div>;
  isLoading?: boolean;
  expandable?: ExpandableComponent<Data>;
  isRowExpandable?: ({ row }: { row: Row<Data> }) => boolean;
  isRowSelectable?: (row: any) => boolean;
  onRowClick: ({
    row,
    isExpanding
  }: {
    row: Row<Data>;
    isExpanding: boolean;
  }) => void;
  onlyOneExpanding?: boolean;
  toggleAllRowsExpanded: (state?: boolean) => void;
  striped?: boolean;
  rowHighlight?: boolean;
  rowIndex: number;
  containerWidth: string | number;
  disabled?: boolean;
  testId?: string;
};

export type GlobalFilter<
  Filter,
  Data extends object = {},
  Props extends object = {}
> = {
  rows?: Row<Data>[];
  preGlobalFilteredRows?: Row<Data>[];
  globalFilter: Filter;
  setGlobalFilter: (filter: Filter) => void;
  toggleAllRowsSelected?: (value?: boolean) => void;
} & Props;

export type TableProps<
  Data extends object = {},
  Filter = never,
  GlobalFilterProps extends object = {}
> = {
  data?: Data[];
  columns: Column<Data>[] | readonly never[];
  rowStyle?: ComponentProps<typeof Div>;
  wrapperStyle?: CSSWithSelectors;
  disableHeaders?: boolean;
  isLoading?: boolean;
  numberOfLoadingRows?: number;
  isEmpty?: boolean;
  renderEmpty?: ({
    globalFilter
  }: {
    globalFilter: FC<GlobalFilter<Filter, Data, GlobalFilterProps>>;
  }) => JSX.Element;
  hasError?: boolean;
  renderError?: () => JSX.Element;
  expandable?: ExpandableComponent<Data>;
  isRowExpandable?: (item: { row: Row<Data> }) => boolean;
  onlyOneExpanding?: boolean;
  onRowSelect?: (item: Data[]) => void;
  isRowSelectable?: (row: Row<Data>) => boolean;
  onSortBy?: (sort: SortingRule<string>[]) => void;
  initialSortBy?: SortingRule<string>[];
  onRowClick?: ({
    row,
    isExpanding
  }: {
    row: Row<Data>;
    isExpanding: boolean;
  }) => void;
  className?: string;
  initialSelectedRowIds?: Record<string, boolean>;
  pageSize?: number;

  GlobalFilter?: FC<GlobalFilter<Filter, Data, GlobalFilterProps>>;
  globalFilterProps?: any;
  filterFunction?: (rows: Row<Data>[], _columns: never, filter: Filter) => void;

  showSelectAllInHeader?: boolean;
  bordered?: boolean;
  onRowMove?: (
    data: Data[],
    {
      newIndex,
      oldIndex
    }: {
      newIndex: number;
      oldIndex: number;
    }
  ) => void;
  disableRowSelect?: boolean;
  getRowId?: (row: Row<Data>) => string;
  selectAllKey?: string;
  clearAllKey?: string;
  allSelectedOnPageKey?: string;
  allSelectedKey?: string;
  selectedCountKey?: string;
  disableSelectAll?: boolean;
  autoResetExpanded?: boolean;
  autoResetPage?: boolean;
} & Omit<React.ComponentProps<typeof Div>, 'columns'>;

export type CellProps<T> = {
  cell: { value: T };
};
